<?php

namespace Drupal\webformautosave\Plugin\rest\resource;

use Drupal\rest\ResourceResponse;
use Drupal\webform\Entity\WebformSubmission;
use Drupal\webform\WebformSubmissionForm;
use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest\ModifiedResourceResponse;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Creates a resource for retrieving webform submission data.
 *
 * @RestResource(
 *   id = "webformautosave_submission",
 *   label = @Translation("Webform Autosave Submission"),
 *   uri_paths = {
 *     "canonical" = "/webformautosave/{webform_id}/submission/{sid}"
 *   }
 * )
 */
class WebformSubmissionResource extends ResourceBase {

  /**
   * Retrieve submission data.
   *
   * @param string $webform_id
   *   Webform ID.
   * @param int $sid
   *   Submission ID.
   *
   * @return \Drupal\rest\ResourceResponse
   *   HTTP response object containing webform submission.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\HttpException
   *   Throws HttpException in case of error.
   */
  public function get($webform_id, $sid) {
    // Verify the webform id and SID are pressent.
    if (empty($webform_id) || empty($sid)) {
      throw new BadRequestHttpException(t('Both webform ID and submission ID are required.'));
    }
    // Load the webform submission.
    /** @var \Drupal\webform\WebformSubmissionInterface $webform_submission */
    $webform_submission = WebformSubmission::load($sid);
    $webform = $webform_submission->getWebform();
    // Check for a submission.
    if (!empty($webform_submission) && !empty($webform) && $webform->id() == $webform_id) {
      // Grab submission data.
      $data = $webform_submission->getData();
      $response_body = [
        'entity' => $webform_submission,
        'data' => $data,
      ];
      // Build the response.
      $response = new ResourceResponse($response_body, 200);
      // Ensure the data is set as our cacheable dependency.
      $response->addCacheableDependency($data);
      // Return the response.
      return $response;
    }
    throw new NotFoundHttpException(t("Can't load webform submission."));

  }
  /**
   * Update submission data.
   *
   * @param string $webform_id
   *   Webform ID.
   * @param int $sid
   *   Submission ID.
   *
   * @param array $webform_data
   *   Webform field data.
   *
   * @return \Drupal\rest\ModifiedResourceResponse
   *   HTTP response object containing webform submission.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\HttpException
   *   Throws HttpException in case of error.
   */
  public function patch($webform_id, $sid, array $webform_data) {
    // Verify the webform id and SID are present.
    if (empty($webform_id) || empty($sid)) {
      throw new BadRequestHttpException(t('Both webform ID and submission ID are required.'));
    }
    // Ensure there is data.
    if (empty($webform_data)) {
      throw new BadRequestHttpException(t('No data has been submitted.'));
    }
    // Load the webform submission.
    /** @var \Drupal\webform\WebformSubmissionInterface $webform_submission */
    $webform_submission = WebformSubmission::load($sid);
    $webform = $webform_submission->getWebform();
    // Check for a submission, webform, and the ID is correct.
    if (!empty($webform_submission) && !empty($webform) && $webform->id() == $webform_id) {
      // Set our submit data
      foreach ($webform_data as $field => $value) {
        $webform_submission->setElementData($field, $value);
      }
      // Put the submission into draft mode.
      $webform_submission->set('in_draft', TRUE);
      // Return submission ID.
      $webform_submission = WebformSubmissionForm::submitWebformSubmission($webform_submission);
      // Take the submission out of draft mode.
      $webform_submission->set('in_draft', FALSE);
      // Build our response.
      $response_body = [
        'sid' => $webform_submission->id(),
        'data' => $webform_submission->getData(),
      ];
      // Return submission ID.
      return new ModifiedResourceResponse($response_body, 200);
    }

    throw new NotFoundHttpException(t("Can't load webform submission."));
  }

}
