<?php

namespace Drupal\webformautosave\Plugin\rest\resource;

use Drupal\webform\Entity\Webform;
use Drupal\webform\WebformSubmissionForm;
use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest\ModifiedResourceResponse;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Symfony\Component\HttpKernel\Exception\HttpException;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Creates a resource for submitting a webform.
 *
 * @RestResource(
 *   id = "webformautosave_submit",
 *   label = @Translation("Webform Autosave Submit"),
 *   uri_paths = {
 *     "canonical" = "/webformautosave/submit",
 *     "https://www.drupal.org/link-relations/create" = "/webformautosave/submit"
 *   }
 * )
 */
class WebformSubmitResource extends ResourceBase {

  /**
   * Responds to entity POST requests and saves the new entity.
   *
   * @param array $webform_data
   *   Webform field data and webform ID.
   *
   * @return \Drupal\rest\ModifiedResourceResponse
   *   The HTTP response object.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\HttpException
   *   Throws HttpException in case of error.
   */
  public function post(array $webform_data) {
    // Basic check for webform ID.
    if (empty($webform_data['webform_id'])) {
      throw new BadRequestHttpException(t("Webform ID wasn't provided"));
    }
    // Convert to webform values format.
    $values = [
      'webform_id' => $webform_data['webform_id'],
      'entity_type' => NULL,
      'entity_id' => NULL,
      'in_draft' => TRUE,
      'uri' => '/webform/' . $webform_data['webform_id'] . '/api',
      'data' => $webform_data,
    ];
    // Don't submit webform ID.
    unset($values['data']['webform_id']);
    /** @var \Drupal\webform\WebformInterface $webform */
    $webform = Webform::load($values['webform_id']);
    // Verify the webform loaded.
    if (!$webform) {
      // Throw an exception because the webform could not be loaded.
      throw new NotFoundHttpException(t("Can't load webform."));
    }
    // Check if the webform is open.
    $is_open = WebformSubmissionForm::isOpen($webform);
    if ($is_open === TRUE) {
      // Return submission ID.
      $webform_submission = WebformSubmissionForm::submitFormValues($values);
      // Take the submission out of draft mode.
      $webform_submission->set('in_draft', FALSE);
      // Build our response.
      $response_body = [
        'sid' => $webform_submission->id(),
        'data' => $webform_submission->getData(),
      ];
      // Return the submission ID.
      return new ModifiedResourceResponse($response_body, 201);
    }
    else {
      // Throw an error if the webform is closed or has too many submissions.
      throw new HttpException(500, 'This webform is closed, or has too many submissions.');
    }
  }

}
